# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging

from designate.tests.functional.api import v2

LOG = logging.getLogger(__name__)


class ApiV2QuotasTest(v2.ApiV2TestCase):
    def setUp(self):
        super().setUp()

    def test_get_quotas(self):
        self.config(quota_api_export_size=1)

        result = self.client.get('/quotas/a', status=200,
                                 headers={'X-Test-Tenant-Id': 'a',
                                          'X-Test-Role': 'member'})

        self.assertEqual(
            {
                'zones': 10,
                'zone_recordsets': 500,
                'zone_records': 500,
                'recordset_records': 20,
                'api_export_size': 1
            },
            result.json
        )

    def test_get_all_quotas(self):
        self.config(quota_zone_recordsets=1)

        result = self.client.get('/quotas', status=200,
                                 headers={'X-Test-Role': 'member'})

        self.assertEqual(
            {
                'api_export_size': 1000,
                'recordset_records': 20,
                'zone_records': 500,
                'zone_recordsets': 1,
                'zones': 10
            },
            result.json
        )

    def test_set_quotas(self):
        self.policy({'set_quota': '@'})

        self.client.patch_json('/quotas/a', {'zones': 123}, status=200,
                               headers={'X-Test-Tenant-Id': 'a',
                                        'X-Test-Role': 'member'})

        result = self.client.get('/quotas/a', status=200,
                                 headers={'X-Test-Tenant-Id': 'a',
                                          'X-Test-Role': 'member'})

        self.assertEqual(
            {
                'zones': 123,
                'zone_recordsets': 500,
                'zone_records': 500,
                'recordset_records': 20,
                'api_export_size': 1000
            },
            result.json
        )

    def test_set_quotas_with_verify_project_id(self):
        self.config(
            quotas_verify_project_id=True,
            group='service:api'
        )

        self.policy({'set_quota': '@'})

        self.client.patch_json('/quotas/a', {'zones': 123}, status=200,
                               headers={'X-Test-Tenant-Id': 'a',
                                        'X-Test-Role': 'member'})

        result = self.client.get('/quotas/a', status=200,
                                 headers={'X-Test-Tenant-Id': 'a',
                                          'X-Test-Role': 'member'})

        self.assertEqual(
            {
                'zones': 123,
                'zone_recordsets': 500,
                'zone_records': 500,
                'recordset_records': 20,
                'api_export_size': 1000
            },
            result.json
        )

    def test_delete_quotas(self):
        self.config(quota_zone_records=1)

        self.policy({'set_quota': '@'})

        # Update recordset_records quota.
        result = self.client.patch_json(
            '/quotas/a', {'recordset_records': 123},
            status=200, headers={'X-Test-Tenant-Id': 'a',
                                 'X-Test-Role': 'member'}
        )
        self.assertEqual(
            {
                'zones': 10,
                'zone_recordsets': 500,
                'zone_records': 1,
                'recordset_records': 123,
                'api_export_size': 1000
            },
            result.json
        )

        # Delete quota.
        self.client.delete('/quotas/a', status=204,
                           headers={'X-Test-Tenant-Id': 'a',
                                    'X-Test-Role': 'member'})

        # Make sure we are back to the default quotas.
        result = self.client.get('/quotas/a', status=200,
                                 headers={'X-Test-Tenant-Id': 'a',
                                          'X-Test-Role': 'member'})

        self.assertEqual(
            {
                'zones': 10,
                'zone_recordsets': 500,
                'zone_records': 1,
                'recordset_records': 20,
                'api_export_size': 1000
            },
            result.json
        )



# Copyright 2015 Rackspace Inc.
#
# Author: Tim Simmons <tim.simmons@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from designate.objects import base
from designate.objects import fields


@base.DesignateRegistry.register
class ZoneExport(base.DictObjectMixin, base.PersistentObjectMixin,
                 base.DesignateObject):
    fields = {
        'status': fields.EnumField(
            nullable=True, valid_values=[
                'ACTIVE', 'PENDING', 'DELETED', 'ERROR', 'COMPLETE'
            ]
        ),
        'task_type': fields.EnumField(
            nullable=True, valid_values=['EXPORT']
        ),
        'tenant_id': fields.StringFields(nullable=True),
        'location': fields.StringFields(nullable=True),
        'message': fields.StringFields(nullable=True, maxLength=160),
        'zone_id': fields.UUIDFields(nullable=True)
    }


@base.DesignateRegistry.register
class ZoneExportList(base.ListObjectMixin, base.DesignateObject,
                     base.PagedListObjectMixin):
    LIST_ITEM_TYPE = ZoneExport

    fields = {
        'objects': fields.ListOfObjectsField('ZoneExport'),
    }



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

import designate.conf
from designate import exceptions
from designate.plugin import DriverPlugin


CONF = designate.conf.CONF


class Quota(DriverPlugin, metaclass=abc.ABCMeta):
    """Base class for quota plugins"""
    __plugin_ns__ = 'designate.quota'
    __plugin_type__ = 'quota'

    def limit_check(self, context, tenant_id, **values):
        resources_exceeded = []
        quotas = self.get_quotas(context, tenant_id)

        for resource, value in values.items():
            if resource in quotas:
                # Setting the resource quota to a negative value will make
                # the resource unlimited
                if quotas[resource] >= 0 and value > quotas[resource]:
                    resources_exceeded.append(resource)
            else:
                raise exceptions.QuotaResourceUnknown(
                    f"'{resource}' is not a valid quota resource."
                )

        if resources_exceeded:
            resources_exceeded.sort(key=len)
            raise exceptions.OverQuota(
                'Quota exceeded for %s.' % ', '.join(resources_exceeded)
            )

    def get_quotas(self, context, tenant_id):
        quotas = self.get_default_quotas(context)
        quotas.update(self._get_quotas(context, tenant_id))

        return quotas

    def get_default_quotas(self, context):
        return {
            'zones': CONF.quota_zones,
            'zone_recordsets': CONF.quota_zone_recordsets,
            'zone_records': CONF.quota_zone_records,
            'recordset_records': CONF.quota_recordset_records,
            'api_export_size': CONF.quota_api_export_size,
        }

    @abc.abstractmethod
    def _get_quotas(self, context, tenant_id):
        """Internal Get Quotas used by get_quotas"""

    @abc.abstractmethod
    def get_quota(self, context, tenant_id, resource):
        """Get Quota"""

    @abc.abstractmethod
    def set_quota(self, context, tenant_id, resource, hard_limit):
        """Set Quota"""

    @abc.abstractmethod
    def reset_quotas(self, context, tenant_id):
        """Reset Quotas"""



# Copyright 2015 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import oslotest.base

from designate.backend import impl_bind9
from designate import context
from designate import exceptions
from designate import objects
from designate.tests import base_fixtures
from designate import utils

import subprocess


class Bind9BackendTestCase(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

        self.admin_context = mock.Mock()
        mock.patch.object(
            context.DesignateContext, 'get_admin_context',
            return_value=self.admin_context).start()

        self.zone = objects.Zone(
            id='cca7908b-dad4-4c50-adba-fb67d4c556e8',
            name='example.com.',
            email='example@example.com'
        )
        self.target = {
            'id': '4588652b-50e7-46b9-b688-a9bad40a873e',
            'type': 'bind9',
            'masters': [
                {'host': '192.0.2.1', 'port': 53},
                {'host': '192.0.2.2', 'port': 35}
            ],
            'options': [
                {'key': 'host', 'value': '192.0.2.3'},
                {'key': 'port', 'value': '53'},
                {'key': 'rndc_host', 'value': '192.0.2.4'},
                {'key': 'rndc_port', 'value': '953'},
                {'key': 'rndc_bin_path', 'value': '/usr/sbin/rndc'},
                {'key': 'rndc_config_file', 'value': '/etc/rndc.conf'},
                {'key': 'rndc_key_file', 'value': '/etc/rndc.key'},
                {'key': 'rndc_timeout', 'value': '0'},
                {'key': 'clean_zonefile', 'value': 'true'}
            ],
        }

        self.backend = impl_bind9.Bind9Backend(
            objects.PoolTarget.from_dict(self.target)
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_create_zone(self, mock_execute):
        with base_fixtures.random_seed(0):
            self.backend.create_zone(self.admin_context, self.zone)

        mock_execute.assert_called_with(
            [
                'addzone',
                'example.com  { type slave; masters { 192.0.2.1 port 53; 192.0.2.2 port 35;}; file "slave.example.com.cca7908b-dad4-4c50-adba-fb67d4c556e8"; };'  # noqa
            ]
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_update_zone(self, mock_execute):
        with base_fixtures.random_seed(0):
            self.backend.update_zone(self.admin_context, self.zone)

        mock_execute.assert_called_with(
            [
                'modzone',
                'example.com  { type slave; masters { 192.0.2.1 port 53; 192.0.2.2 port 35;}; file "slave.example.com.cca7908b-dad4-4c50-adba-fb67d4c556e8"; };'  # noqa
            ]
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    @mock.patch.object(impl_bind9.Bind9Backend, 'get_zone')
    def test_update_zone_error(self, mock_get_zone, mock_execute):
        mock_get_zone.return_value = True
        mock_execute.side_effect = exceptions.Backend('error')

        with base_fixtures.random_seed(0):
            self.backend.update_zone(self.admin_context, self.zone)

        self.assertIn('Error updating zone', self.stdlog.logger.output)

    @mock.patch.object(impl_bind9.Bind9Backend, 'create_zone')
    @mock.patch.object(impl_bind9.Bind9Backend, 'get_zone')
    def test_update_zone_does_not_exist(self, mock_get_zone, mock_create_zone):
        mock_get_zone.return_value = False

        self.backend.update_zone(self.admin_context, self.zone)

        mock_create_zone.assert_called()

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_get_zone(self, mock_execute):
        with base_fixtures.random_seed(0):
            self.backend.get_zone(self.admin_context, self.zone)

        mock_execute.assert_called_with(
            ['showzone', 'example.com ']
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_get_zone_backend_error(self, mock_execute):
        mock_execute.side_effect = exceptions.Backend('error')

        self.assertRaisesRegex(
            exceptions.Backend,
            'error',
            self.backend.get_zone, self.admin_context, self.zone
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_get_zone_backend_error_not_found(self, mock_execute):
        mock_execute.side_effect = exceptions.Backend('not found')

        self.assertFalse(self.backend.get_zone(self.admin_context, self.zone))

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_create_zone_with_view(self, mock_execute):
        self.target['options'].append(
            {'key': 'view', 'value': 'guest'},
        )

        backend = impl_bind9.Bind9Backend(
            objects.PoolTarget.from_dict(self.target)
        )

        with base_fixtures.random_seed(1):
            backend.create_zone(self.admin_context, self.zone)

        mock_execute.assert_called_with(
            [
                'addzone',
                'example.com in guest { type slave; masters { 192.0.2.2 port 35; 192.0.2.1 port 53;}; file "slave.example.com.cca7908b-dad4-4c50-adba-fb67d4c556e8"; };'  # noqa
            ]
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_create_zone_raises_on_exception(self, mock_execute):
        mock_execute.side_effect = exceptions.Backend('badop')
        self.assertRaises(
            exceptions.Backend,
            self.backend.create_zone, self.admin_context, self.zone
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_create_zone_already_exists(self, mock_execute):
        mock_execute.side_effect = exceptions.Backend('already exists')

        self.backend.create_zone(self.admin_context, self.zone)

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_delete_zone(self, mock_execute):
        self.backend.delete_zone(self.admin_context, self.zone)

        mock_execute.assert_called_with(
            ['delzone', '-clean', 'example.com ']
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_delete_zone_with_view(self, mock_execute):
        self.target['options'].append(
            {'key': 'view', 'value': 'guest'},
        )

        backend = impl_bind9.Bind9Backend(
            objects.PoolTarget.from_dict(self.target)
        )

        backend.delete_zone(self.admin_context, self.zone)

        mock_execute.assert_called_with(
            ['delzone', '-clean', 'example.com in guest']
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_delete_zone_without_clean_zonefile(self, mock_execute):
        self.target['options'] = [
            {'key': 'clean_zonefile', 'value': 'false'}
        ]

        backend = impl_bind9.Bind9Backend(
            objects.PoolTarget.from_dict(self.target)
        )

        backend.delete_zone(self.admin_context, self.zone, {})

        mock_execute.assert_called_with(
            ['delzone', 'example.com ']
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_delete_zone_raises_on_exception(self, mock_execute):
        mock_execute.side_effect = exceptions.Backend('badop')
        self.assertRaises(
            exceptions.Backend,
            self.backend.delete_zone, self.admin_context, self.zone
        )

    @mock.patch.object(impl_bind9.Bind9Backend, '_execute_rndc')
    def test_delete_zone_already_deleted(self, mock_execute):
        mock_execute.side_effect = exceptions.Backend('not found')

        self.backend.delete_zone(self.admin_context, self.zone)

    def test_generate_rndc_base_call(self):
        self.assertEqual(
            [
                '/usr/sbin/rndc', '-s', '192.0.2.4', '-p', '953',
                '-c', '/etc/rndc.conf', '-k', '/etc/rndc.key'
            ],
            self.backend._generate_rndc_base_call()
        )

    def test_generate_rndc_base_call_without_config_file(self):
        self.target['options'] = [
            {'key': 'rndc_host', 'value': '192.0.2.4'},
            {'key': 'rndc_port', 'value': '953'},
            {'key': 'rndc_bin_path', 'value': '/usr/sbin/rndc'},
            {'key': 'rndc_key_file', 'value': '/etc/rndc.key'},
        ]

        backend = impl_bind9.Bind9Backend(
            objects.PoolTarget.from_dict(self.target)
        )

        self.assertEqual(
            [
                '/usr/sbin/rndc', '-s', '192.0.2.4', '-p', '953',
                '-k', '/etc/rndc.key'
            ],
            backend._generate_rndc_base_call()
        )

    def test_generate_rndc_base_call_without_key_file(self):
        self.target['options'] = [
            {'key': 'rndc_host', 'value': '192.0.2.4'},
            {'key': 'rndc_port', 'value': '953'},
            {'key': 'rndc_bin_path', 'value': '/usr/sbin/rndc'},
            {'key': 'rndc_config_file', 'value': '/etc/rndc.conf'},
        ]

        backend = impl_bind9.Bind9Backend(
            objects.PoolTarget.from_dict(self.target)
        )

        self.assertEqual(
            [
                '/usr/sbin/rndc', '-s', '192.0.2.4', '-p', '953',
                '-c', '/etc/rndc.conf'
            ],
            backend._generate_rndc_base_call()
        )

    @mock.patch('designate.utils.execute')
    def test_execute_rndc(self, mock_execute):
        rndc_op = ['delzone', 'example.com ']

        self.backend._execute_rndc(rndc_op)

        mock_execute.assert_called_with(
            '/usr/sbin/rndc', '-s', '192.0.2.4', '-p', '953',
            '-c', '/etc/rndc.conf', '-k', '/etc/rndc.key',
            'delzone', 'example.com ', timeout=None
        )

    @mock.patch('designate.utils.execute')
    def test_execute_rndc_timeout(self, mock_execute):
        rndc_op = ['delzone', 'example.com ']

        self.backend._rndc_timeout = 10
        self.backend._execute_rndc(rndc_op)

        mock_execute.assert_called_with(
            '/usr/sbin/rndc', '-s', '192.0.2.4', '-p', '953',
            '-c', '/etc/rndc.conf', '-k', '/etc/rndc.key',
            'delzone', 'example.com ', timeout=10
        )

    @mock.patch('designate.utils.execute')
    def test_execute_rndc_timeout_exception(self, mock_execute):
        rndc_op = ['delzone', 'example.com ']

        self.backend._rndc_timeout = 10

        mock_execute.side_effect = subprocess.TimeoutExpired([
            '/usr/sbin/rndc', '-s', '192.0.2.4', '-p', '953',
            '-c', '/etc/rndc.conf', '-k', '/etc/rndc.key',
            'delzone', 'example.com '], 10)

        self.assertRaises(
            exceptions.Backend,
            self.backend._execute_rndc, rndc_op
        )

    @mock.patch('designate.utils.execute')
    def test_execute_rndc_raises_on_exception(self, mock_execute):
        mock_execute.side_effect = utils.processutils.ProcessExecutionError()
        rndc_op = ['badop', 'example.com ']

        self.assertRaises(
            exceptions.Backend,
            self.backend._execute_rndc, rndc_op
        )



# Copyright 2016 Rackspace Inc.
#
# Author: Eric Larson <eric.larson@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading


from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslo_messaging as messaging
import oslotest.base

from designate import backend
from designate.central import rpcapi as central_rpcapi
import designate.conf
from designate import exceptions
from designate import objects
from designate import policy
from designate import rpc
import designate.service
from designate import storage
import designate.tests
from designate.tests import base_fixtures
from designate.worker import processing
from designate.worker import service


CONF = designate.conf.CONF


class WorkerServiceTest(oslotest.base.BaseTestCase):
    @mock.patch.object(policy, 'init')
    @mock.patch.object(rpc, 'get_client')
    @mock.patch.object(rpc, 'init')
    @mock.patch.object(rpc, 'initialized')
    def setUp(self, mock_rpc_initialized, mock_rpc_init, mock_rpc_get_client,
              mock_policy_init):
        super().setUp()

        mock_rpc_initialized.return_value = False

        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)
        self.useFixture(cfg_fixture.Config(CONF))

        self.context = mock.Mock()
        self.zone = mock.Mock()
        self.service = service.Service()

        mock_policy_init.assert_called_once()
        mock_rpc_initialized.assert_called_once()
        mock_rpc_init.assert_called_once()

    @mock.patch.object(designate.service.RPCService, 'start')
    def test_service_start(self, mock_rpc_start):
        self.service.start()

        self.assertTrue(mock_rpc_start.called)

    @mock.patch.object(designate.rpc, 'get_notification_listener')
    def test_service_stop(self, mock_notification_listener):
        self.service.stop()

        self.assertIn('Stopping worker service', self.stdlog.logger.output)

    def test_service_name(self):
        self.assertEqual('worker', self.service.service_name)

    @mock.patch.object(policy, 'init', mock.Mock())
    @mock.patch.object(rpc, 'get_client', mock.Mock())
    @mock.patch.object(rpc, 'init', mock.Mock())
    @mock.patch.object(rpc, 'initialized', mock.Mock(return_value=False))
    def test_worker_rpc_topic(self):
        CONF.set_override('topic', 'test-topic', 'service:worker')

        self.service = service.Service()

        self.assertEqual('test-topic', self.service.rpc_topic)
        self.assertEqual('worker', self.service.service_name)

    @mock.patch.object(rpc, 'get_client', mock.Mock())
    def test_central_api(self):
        self.assertIsNone(self.service._central_api)
        self.assertIsInstance(self.service.central_api,
                              central_rpcapi.CentralAPI)
        self.assertIsNotNone(self.service._central_api)
        self.assertIsInstance(self.service.central_api,
                              central_rpcapi.CentralAPI)

    @mock.patch.object(storage, 'get_storage', mock.Mock())
    def test_storage(self):
        self.assertIsNone(self.service._storage)
        self.assertIsInstance(self.service.storage, mock.Mock)
        self.assertIsNotNone(self.service._storage)
        self.assertIsInstance(self.service.storage, mock.Mock)

    @mock.patch.object(backend, 'get_backend', mock.Mock())
    def test_setup_target_backends(self):
        pool = objects.Pool.from_dict({
            'targets': [
                {
                    'description': 'Fake DNS Cluster',
                    'masters': [],
                    'options': [],
                    'type': 'fake',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                }
            ]
        })

        self.service._setup_target_backends(pool)

    def test_setup_target_backends_with_no_backends(self):
        pool = objects.Pool.from_dict({
            'targets': []
        })

        self.assertRaises(
            exceptions.NoPoolTargetsConfigured,
            self.service._setup_target_backends, pool
        )

    @mock.patch.object(backend, 'get_backend', mock.Mock())
    def test_load_pool(self):
        pool = objects.Pool.from_dict({
            'targets': [
                {
                    'description': 'Fake DNS Cluster',
                    'masters': [],
                    'options': [],
                    'type': 'fake',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                }
            ]
        })

        self.service._central_api = mock.MagicMock()
        self.service._central_api.get_pool.return_value = pool
        self.assertIsInstance(
            self.service.load_pool('cf2e8eab-76cd-4162-bf76-8aeee3556de0'),
            objects.Pool
        )

    @mock.patch('time.sleep', mock.Mock())
    @mock.patch.object(backend, 'get_backend', mock.Mock())
    def test_load_pool_with_failures(self):
        pool_no_targets = objects.Pool.from_dict({
            'targets': [],
        })

        pool_success = objects.Pool.from_dict({
            'targets': [
                {
                    'description': 'Fake DNS Cluster',
                    'masters': [],
                    'options': [],
                    'type': 'fake',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                }
            ]
        })

        self.service._central_api = mock.MagicMock()
        self.service._central_api.get_pool.side_effect = [
            messaging.exceptions.MessagingTimeout,
            exceptions.PoolNotFound,
            pool_no_targets,
            pool_success
        ]

        self.assertIsInstance(
            self.service.load_pool('cf2e8eab-76cd-4162-bf76-8aeee3556de0'),
            objects.Pool
        )

    def test_create_zone(self):
        self.service._do_zone_action = mock.Mock()

        self.service.create_zone(self.context, self.zone)

        self.service._do_zone_action.assert_called_with(
            self.context, self.zone
        )

    def test_get_executor(self):
        self.assertIsInstance(self.service.executor, processing.Executor)

    def test_get_pools_map(self):
        self.assertIsInstance(self.service.pools_map, dict)

    def test_delete_zone(self):
        self.service._do_zone_action = mock.Mock()
        self.zone_params = {}

        self.service.delete_zone(self.context, self.zone)

        self.service._do_zone_action.assert_called_with(
            self.context, self.zone, self.zone_params
        )

    def test_delete_zone_hard(self):
        self.service._do_zone_action = mock.Mock()
        self.zone_params = {'hard_delete': True}

        self.service.delete_zone(self.context, self.zone, hard_delete=True)

        self.service._do_zone_action.assert_called_with(
            self.context, self.zone, self.zone_params
        )

    def test_update_zone(self):
        self.service._do_zone_action = mock.Mock()

        self.service.update_zone(self.context, self.zone)

        self.service._do_zone_action.assert_called_with(
            self.context, self.zone
        )

    @mock.patch.object(service.zonetasks, 'ZoneAction')
    def test_do_zone_action(self, mock_zone_action):
        self.service._executor = mock.Mock()
        self.service._pool = mock.Mock()
        self.service.get_pool = mock.Mock()
        pool = mock.Mock()
        pool.also_notifies = mock.MagicMock()
        pool.also_notifies.__iter__.return_value = []
        self.service.get_pool.return_value = pool
        self.zone_params = {}

        self.service._do_zone_action(self.context, self.zone,
                                     self.zone_params)

        mock_zone_action.assert_called_with(
            self.service.executor,
            self.context,
            pool,
            self.zone,
            self.zone.action,
            self.zone_params
        )

        self.service._executor.run.assert_called_with([mock_zone_action()])

    @mock.patch.object(service.zonetasks, 'ZoneAction')
    @mock.patch.object(service.zonetasks, 'SendNotify')
    def test_do_zone_action_also_notifies(self, mock_send_notify,
                                          mock_zone_action):
        self.service._executor = mock.Mock()
        self.service._pool = mock.Mock()
        self.service.get_pool = mock.Mock()
        pool = mock.Mock()
        pool.also_notifies = mock.MagicMock()
        pool.also_notifies.__iter__.return_value = [
            mock.Mock(host='192.0.2.1', port=53),
        ]
        self.service.get_pool.return_value = pool
        self.zone_params = {}

        self.service._do_zone_action(self.context, self.zone,
                                     self.zone_params)

        mock_zone_action.assert_called_with(
            self.service.executor,
            self.context,
            pool,
            self.zone,
            self.zone.action,
            self.zone_params
        )

        self.service._executor.run.assert_called_with(
            [mock_zone_action(), mock_send_notify()]
        )

    def test_get_pool(self):
        pool = mock.Mock()
        self.service.load_pool = mock.Mock()
        self.service.load_pool.return_value = pool
        self.service._pools_map = {'1': pool}

        self.assertEqual(pool, self.service.get_pool('1'))
        self.assertEqual(pool, self.service.get_pool('2'))

    @mock.patch.object(service.zonetasks, 'RecoverShard')
    def test_recover_shard(self, mock_recover_shard):
        self.service._executor = mock.Mock()
        self.service._pool = mock.Mock()

        self.service.recover_shard(self.context, 1, 10)

        mock_recover_shard.assert_called_with(
            self.service.executor,
            self.context,
            1, 10
        )

        self.service.executor.run.assert_called_with(mock_recover_shard())

    @mock.patch.object(service.zonetasks, 'ExportZone')
    def test_start_zone_export(self, mock_export_zone):
        self.service._executor = mock.Mock()
        self.service._pool = mock.Mock()
        zone = mock.Mock()
        export = mock.Mock()

        self.service.start_zone_export(self.context, zone, export)

        mock_export_zone.assert_called_with(
            self.service.executor,
            self.context,
            zone,
            export
        )

        self.service.executor.run.assert_called_with(mock_export_zone())

    @mock.patch.object(service.zonetasks, 'ZoneXfr')
    def test_perform_zone_xfr(self, mock_perform_zone_xfr):
        self.service._executor = mock.Mock()
        self.service._pool = mock.Mock()
        zone = mock.Mock()

        self.service.perform_zone_xfr(self.context, zone)

        mock_perform_zone_xfr.assert_called_with(
            self.service.executor,
            self.context,
            zone,
            None
        )

        self.service.executor.run.assert_called_with(mock_perform_zone_xfr())

    @mock.patch.object(service.zonetasks, 'GetZoneSerial')
    def test_get_serial_number(self, mock_get_serial_number):
        zone = mock.Mock()

        self.service.get_serial_number(
            self.context, zone, '203.0.113.1', 53
        )

        mock_get_serial_number.assert_called_with(
            self.service.executor,
            self.context,
            zone,
            '203.0.113.1',
            53
        )



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from urllib import parse

import designate.conf
from designate import exceptions
from designate.objects.adapters import base
from designate.objects import base as ovoobj_base


CONF = designate.conf.CONF


class APIv2Adapter(base.DesignateAdapter):
    ADAPTER_FORMAT = 'API_v2'

    #####################
    # Rendering methods #
    #####################

    @classmethod
    def render_list(cls, list_objects, *args, **kwargs):
        r_list = super().render_list(
            list_objects, *args, **kwargs)

        if (cls.MODIFICATIONS['options'].get('links', True) and
                'request' in kwargs):
            r_list['links'] = cls._get_collection_links(
                list_objects, kwargs['request']
            )
        # Check if we should include metadata
        if isinstance(list_objects, ovoobj_base.PagedListObjectMixin):
            metadata = {}
            if list_objects.total_count is not None:
                metadata['total_count'] = list_objects.total_count
            r_list['metadata'] = metadata

        return r_list

    @classmethod
    def render_object(cls, obj, *args, **kwargs):
        new_obj = super().render_object(obj, *args, **kwargs)

        if (cls.MODIFICATIONS['options'].get('links', True) and
                'request' in kwargs):
            new_obj['links'] = cls._get_resource_links(obj, kwargs['request'])

        return new_obj

    #####################
    #  Parsing methods  #
    #####################

    @classmethod
    def parse(cls, values, output_object, *args, **kwargs):
        return super().parse(
            cls.ADAPTER_FORMAT, values, output_object, *args, **kwargs)

    #####################
    #    Link methods   #
    #####################

    @classmethod
    def _get_base_url(cls, request):
        if CONF['service:api'].enable_host_header:
            return request.host_url
        return CONF['service:api'].api_base_uri.rstrip('/')

    @classmethod
    def _get_resource_links(cls, obj, request):
        base_uri = cls._get_base_url(request)

        path = cls._get_path(request, obj)
        return {'self': f'{base_uri}{path}/{obj.id}'}

    @classmethod
    def _get_path(cls, request, *args):
        path = request.path.lstrip('/').split('/')
        item_path = ''
        for part in path:
            if part == cls.MODIFICATIONS['options']['collection_name']:
                item_path += '/' + part
                return item_path
            else:
                item_path += '/' + part

    @classmethod
    def _get_collection_links(cls, item_list, request):

        links = {
            'self': cls._get_collection_href(request)
        }
        params = request.GET

        # defined in etc/designate/designate.conf.sample
        limit = CONF['service:api'].default_limit_v2

        if 'limit' in params:
            limit = params['limit']
            if limit.lower() == 'max':
                limit = CONF['service:api'].max_limit_v2
            else:
                try:
                    limit = int(limit)
                except ValueError:
                    raise exceptions.ValueError(
                        "'limit' should be an integer or 'max'")

        # Bug: this creates a link to "next" even on the last page if
        # len(item_list) happens to be == limit
        if limit is not None and limit == len(item_list):
            links['next'] = cls._get_next_href(request, item_list)

        return links

    @classmethod
    def _get_collection_href(cls, request, extra_params=None):
        params = request.GET

        if extra_params is not None:
            params.update(extra_params)

        base_uri = cls._get_base_url(request)

        href = '{}{}?{}'.format(
            base_uri,
            cls._get_path(request),
            parse.urlencode(params))

        return href.rstrip('?')

    @classmethod
    def _get_next_href(cls, request, items):
        # Prepare the extra params
        extra_params = {
            'marker': items[-1]['id']
        }

        return cls._get_collection_href(request, extra_params)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Add increment serial

Revision ID: a005af3aa38e
Revises: b20189fd288e
Create Date: 2023-01-21 17:39:00.822775

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'a005af3aa38e'
down_revision = 'b20189fd288e'
branch_labels = None
depends_on = None


def upgrade() -> None:
    op.add_column(
        'zones',
        sa.Column('increment_serial', sa.Boolean, default=False)
    )
    op.create_index(
        'increment_serial', 'zones', ['increment_serial']
    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

import oslotest.base

from designate.api.v2.controllers.zones.tasks import pool_move
from designate.central import rpcapi
from designate import exceptions
from designate import objects


class TestPoolMoveAPI(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.central_api = mock.Mock()
        self.zone = objects.Zone(
            id='c2c6381f-77f6-4e63-a63d-fda0fc22b0b2',
            pool_id='bddce92d-5ca5-40e0-b787-c1ba9884278f',
            name='example.com.',
            email='example@example.com'
        )
        mock.patch.object(rpcapi.CentralAPI, 'get_instance',
                          return_value=self.central_api).start()

        self.controller = pool_move.PoolMoveController()

    @mock.patch('pecan.response', mock.Mock())
    @mock.patch('pecan.request')
    def test_post_all_target_pool_not_different(self, mock_request):
        mock_request.environ = {'context': mock.Mock()}
        mock_request.body_dict = {
            'pool_id': 'bddce92d-5ca5-40e0-b787-c1ba9884278f'
        }
        mock_pool_move = mock.Mock()
        mock_pool_move.status = 'ERROR'

        self.central_api.get_zone.return_value = self.zone
        self.central_api.pool_move_zone.return_value = mock_pool_move

        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Target pool must be different for zone pool move',
            self.controller.post_all, self.zone.id
        )

    @mock.patch('pecan.response')
    @mock.patch('pecan.request')
    def test_post_all_move_error(self, mock_request, mock_response):
        mock_request.environ = {'context': mock.Mock()}
        mock_pool_move = mock.Mock()
        mock_pool_move.status = 'ERROR'

        self.central_api.get_zone.return_value = self.zone
        self.central_api.pool_move_zone.return_value = mock_pool_move

        self.controller.post_all(self.zone.id)

        self.assertEqual(500, mock_response.status_int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading


from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
from designate.notification_handler import nova
from designate.tests import base_fixtures


CONF = designate.conf.CONF


class TestNovaHandler(oslotest.base.BaseTestCase):
    @mock.patch('designate.rpc.get_client', mock.Mock())
    def setUp(self):
        super().setUp()

        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)
        self.useFixture(cfg_fixture.Config(CONF))

        self.zone_id = 'b0dc7c26-f605-41c0-b8aa-65d7c086495f'

        CONF.set_override(
            'enabled_notification_handlers',
            [nova.NovaFixedHandler.__plugin_name__],
            'service:sink'
        )
        CONF.set_override(
            'zone_id', self.zone_id, 'handler:nova_fixed'
        )

        self.handler = nova.NovaFixedHandler()
        self.handler._create = mock.Mock()
        self.handler._delete = mock.Mock()

    def test_get_name(self):
        self.assertEqual(
            self.handler.name,
            'handler:nova_fixed'
        )

    def test_get_canonical_name(self):
        self.assertEqual(
            self.handler.get_canonical_name(),
            'handler:nova_fixed'
        )

    def test_get_exchange_topics(self):
        self.assertEqual(
            self.handler.get_exchange_topics(),
            ('nova', ['notifications'])
        )

    def test_get_event_types(self):
        self.assertEqual(
            self.handler.get_event_types(),
            [
                'compute.instance.create.end',
                'compute.instance.delete.start'
            ]
        )

    def test_process_notification_create(self):
        payload = {
            'fixed_ips': mock.Mock(),
            'instance_id': mock.Mock(),
        }
        self.handler.process_notification(
            {}, 'compute.instance.create.end', payload
        )

        self.handler._create.assert_called_with(
            addresses=mock.ANY,
            extra={
                'fixed_ips': mock.ANY,
                'instance_id': mock.ANY,
                'project': None
            },
            zone_id=self.zone_id,
            resource_id=mock.ANY,
            resource_type='instance'
        )
        self.handler._delete.assert_not_called()

    def test_process_notification_delete(self):
        payload = {
            'instance_id': mock.Mock(),
        }
        self.handler.process_notification(
            {}, 'compute.instance.delete.start', payload
        )

        self.handler._create.assert_not_called()
        self.handler._delete.assert_called_with(
            zone_id=self.zone_id,
            resource_id=mock.ANY,
            resource_type='instance'
        )

    def test_process_notification_invalid_event_type(self):
        self.handler.process_notification(
            mock.Mock(), 'compute.instance.delete.end', None
        )

        self.handler._create.assert_not_called()
        self.handler._delete.assert_not_called()

    def test_process_notification_no_zone_id_set(self):
        CONF.set_override('zone_id', None, 'handler:nova_fixed')

        self.handler.process_notification(
            mock.Mock(), 'compute.instance.create.end', None
        )

        self.assertIn(
            'NovaFixedHandler: zone_id is None, ignore the event.',
            self.stdlog.logger.output
        )

        self.handler._create.assert_not_called()
        self.handler._delete.assert_not_called()



# Copyright (c) 2014 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from designate.common import constants
from designate.objects import base
from designate.objects import fields


@base.DesignateRegistry.register
class Quota(base.DictObjectMixin, base.PersistentObjectMixin,
            base.DesignateObject):
    fields = {
        'tenant_id': fields.AnyField(nullable=True),
        'resource': fields.EnumField(nullable=True,
                                     valid_values=constants.VALID_QUOTAS),
        'hard_limit': fields.IntegerFields(nullable=True,
                                           minimum=constants.MIN_QUOTA,
                                           maximum=constants.MAX_QUOTA)
    }

    STRING_KEYS = [
        'resource', 'tenant_id', 'hard_limit'
    ]


@base.DesignateRegistry.register
class QuotaList(base.ListObjectMixin, base.DesignateObject):
    LIST_ITEM_TYPE = Quota

    fields = {
        'objects': fields.ListOfObjectsField('Quota'),
    }

    @classmethod
    def from_dict(cls, _dict):
        instance = cls()
        for field, value in _dict.items():
            item = cls.LIST_ITEM_TYPE()
            item.resource = field
            item.hard_limit = value
            instance.append(item)

        return instance

    def to_dict(self):
        _dict = {}
        for quota in self.objects:
            _dict[quota.resource] = quota.hard_limit
        return _dict



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import binascii
from unittest import mock

import dns
import dns.rdataclass
import dns.rdatatype
import dns.resolver
import dns.rrset
import dns.tsigkeyring
import testtools

import designate.conf
from designate import context
from designate.mdns import handler
from designate import objects
import designate.tests.functional


CONF = designate.conf.CONF
default_pool_id = CONF['service:central'].default_pool_id

ANSWER = [
    "id 1234",
    "opcode QUERY",
    "rcode NOERROR",
    "flags QR AA RD",
    ";QUESTION",
    "example.com. IN SOA",
    ";ANSWER",
    "example.com. 3600 IN SOA ns1.example.com. root.master.com. "
    "%(serial)s 3600 1800 604800 3600",
    ";AUTHORITY",
    "example.com. 3600 IN NS ns1.master.com.",
    ";ADDITIONAL"
]


class MdnsRequestHandlerTest(designate.tests.functional.TestCase):
    def setUp(self):
        super().setUp()
        self.mock_tg = mock.Mock()
        self.handler = handler.RequestHandler(self.storage, self.mock_tg)
        self.addr = ["0.0.0.0", 5556]

        self.context = context.DesignateContext.get_admin_context(
            all_tenants=True)

        # Create a TSIG Key for the default pool, and another for some other
        # pool.
        self.tsigkey_pool_default = self.create_tsigkey(
            name='default-pool',
            scope='POOL',
            resource_id=default_pool_id)

        self.tsigkey_pool_unknown = self.create_tsigkey(
            name='unknown-pool',
            scope='POOL',
            resource_id='628e55a0-c724-4767-8c59-0a61c15d3444')

        self.tsigkey_zone_unknown = self.create_tsigkey(
            name='unknown-zone',
            scope='ZONE',
            resource_id='82fd08be-9eb7-4d94-8267-a26f8348671d')

    def test_dispatch_opcode_iquery(self):
        # DNS packet with IQUERY opcode
        payload = "271109000001000000000000076578616d706c6503636f6d0000010001"

        # expected response is an error code REFUSED.  The other fields are
        # id 10001
        # opcode IQUERY
        # rcode REFUSED
        # flags QR RD
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"271189050001000000000000076578616d706c6503636f"
                             b"6d0000010001")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_status(self):
        # DNS packet with STATUS opcode
        payload = "271211000001000000000000076578616d706c6503636f6d0000010001"

        # expected response is an error code REFUSED.  The other fields are
        # id 10002
        # opcode STATUS
        # rcode REFUSED
        # flags QR RD
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"271291050001000000000000076578616d706c6503636f"
                             b"6d0000010001")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def _get_secondary_zone(self, values=None, attributes=None,
                            masters=None):
        attributes = attributes or []
        masters = masters or [{"host": "192.0.2.1", "port": 53}]
        fixture = self.get_zone_fixture("SECONDARY", values=values)
        fixture['email'] = CONF['service:central'].managed_resource_email

        zone = objects.Zone(**fixture)
        zone.attributes = objects.ZoneAttributeList().from_list(attributes)
        zone.masters = objects.ZoneMasterList().from_list(masters)
        return zone

    def _get_soa_answer(self, serial):
        text = "\n".join(ANSWER) % {"serial": str(serial)}
        msg = dns.message.from_text(text)
        name = dns.name.from_text('example.com.')
        answer = dns.resolver.Answer(name, dns.rdatatype.SOA,
                                     dns.rdataclass.IN, msg)
        return answer

    @mock.patch.object(dns.resolver.Resolver, 'resolve')
    def test_dispatch_opcode_notify_different_serial(self, func):
        # DNS packet with NOTIFY opcode
        payload = "c38021000001000000000000076578616d706c6503636f6d0000060001"

        master = "192.0.2.1"
        zone = self._get_secondary_zone({"serial": 123})

        # expected response is an error code NOERROR.  The other fields are
        # id 50048
        # opcode NOTIFY
        # rcode NOERROR
        # flags QR AA RD
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"c380a5000001000000000000076578616d706c6503636f"
                             b"6d0000060001")

        # The SOA serial should be different from the one in thezone and
        # will trigger a AXFR
        func.return_value = self._get_soa_answer(123123)

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {
            'addr': (master, 53),
            'context': self.context
        }

        with mock.patch.object(self.handler.storage, 'find_zone',
                               return_value=zone):
            response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    @mock.patch.object(dns.resolver.Resolver, 'resolve')
    def test_dispatch_opcode_notify_same_serial(self, func):
        # DNS packet with NOTIFY opcode
        payload = "c38021000001000000000000076578616d706c6503636f6d0000060001"

        master = "192.0.2.1"
        zone = self._get_secondary_zone({"serial": 123})

        # expected response is an error code NOERROR.  The other fields are
        # id 50048
        # opcode NOTIFY
        # rcode NOERROR
        # flags QR AA RD
        # ;QUESTION
        # example.com. IN SOA
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"c380a5000001000000000000076578616d706c6503636f"
                             b"6d0000060001")

        # The SOA serial should be different from the one in thezone and
        # will trigger a AXFR
        func.return_value = self._get_soa_answer(zone.serial)

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {
            'addr': (master, 53),
            'context': self.context
        }

        with mock.patch.object(self.handler.storage, 'find_zone',
                               return_value=zone):
            response = next(self.handler(request)).to_wire()

        assert not self.mock_tg.add_thread.called
        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_notify_invalid_master(self):
        # DNS packet with NOTIFY opcode
        payload = "c38021000001000000000000076578616d706c6503636f6d0000060001"

        # Have a zone with different master then the one where the notify
        # comes from causing it to be "ignored" as in not transferred and
        # logged

        zone = self._get_secondary_zone({"serial": 123})

        # expected response is an error code REFUSED.  The other fields are
        # id 50048
        # opcode NOTIFY
        # rcode REFUSED
        # flags QR AA RD
        # ;QUESTION
        # example.com. IN SOA
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"c380a1050001000000000000076578616d706c6503636f"
                             b"6d0000060001")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {
            'addr': ("192.0.2.2", 53),
            'context': self.context
        }

        with mock.patch.object(self.handler.storage, 'find_zone',
                               return_value=zone):
            response = next(self.handler(request)).to_wire()

        assert not self.mock_tg.add_thread.called
        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_notify_no_question_formerr(self):
        # DNS packet with NOTIFY opcode and no question
        payload = "f16320000000000000000000"

        # expected response is an error code FORMERR.  The other fields are
        # id 61795
        # opcode NOTIFY
        # rcode FORMERR
        # flags QR RD
        # ;QUESTION
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"f163a0010000000000000000")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {
            'addr': ("192.0.2.2", 53),
            'context': self.context
        }

        response = next(self.handler(request)).to_wire()

        assert not self.mock_tg.add_thread.called
        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_notify_invalid_zone(self):
        # DNS packet with NOTIFY opcode
        payload = "c38021000001000000000000076578616d706c6503636f6d0000060001"

        # expected response is an error code NOTAUTH.  The other fields are
        # id 50048
        # opcode NOTIFY
        # rcode NOTAUTH
        # flags QR RD
        # ;QUESTION
        # example.com. IN SOA
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"c380a1090001000000000000076578616d706c6503636f"
                             b"6d0000060001")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {
            'addr': ("192.0.2.2", 53),
            'context': self.context
        }

        response = next(self.handler(request)).to_wire()

        assert not self.mock_tg.add_thread.called
        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_update(self):
        # DNS packet with UPDATE opcode
        payload = "271429000001000000000000076578616d706c6503636f6d0000060001"

        # expected response is an error code REFUSED.  The other fields are
        # id 10004
        # opcode UPDATE
        # rcode REFUSED
        # flags QR RD
        # ;ZONE
        # example.com. IN SOA
        # ;PREREQ
        # ;UPDATE
        # ;ADDITIONAL
        expected_response = (b"2714a9050001000000000000076578616d706c6503636f"
                             b"6d0000060001")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    # def test_dispatch_opcode_query_invalid(self):
    #     # invalid query
    #     payload = "1234"

    #     # expected_response is FORMERR.  The other fields are
    #     # id <varies>
    #     # opcode QUERY
    #     # rcode FORMERR
    #     # flags QR RD
    #     # ;QUESTION
    #     # ;ANSWER
    #     # ;AUTHORITY
    #     # ;ADDITIONAL
    #     expected_response = "1010000000000000000"

    #     request = dns.message.from_wire(binascii.a2b_hex(payload))
    #     request.environ = {'addr': self.addr, 'context': self.context}
    #     response = self.handler(request).next().to_wire()

    #     # strip the id from the response and compare
    #     self.assertEqual(expected_response, binascii.b2a_hex(response)[5:])

    def test_dispatch_opcode_query_non_existent_zone(self):
        # DNS packet with QUERY opcode
        # query is for example.com. IN A
        payload = ("271501200001000000000001076578616d706c6503636f6d0000010001"
                   "0000291000000000000000")

        # expected_response is an error code REFUSED.  The other fields are
        # id 10005
        # opcode QUERY
        # rcode REFUSED
        # flags QR RD
        # edns 0
        # payload 8192
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"271581050001000000000001076578616d706c6503636f"
                             b"6d00000100010000292000000000000000")
        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_A(self):
        # query is for mail.example.com. IN A
        payload = ("271601000001000000000000046d61696c076578616d706c6503636f6d"
                   "0000010001")

        # expected_response is NOERROR.  The other fields are
        # id 10006
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA RD
        # ;QUESTION
        # mail.example.com. IN A
        # ;ANSWER
        # mail.example.com. 3600 IN A 192.0.2.1
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"271685000001000100000000046d61696c076578616d70"
                             b"6c6503636f6d0000010001c00c0001000100000e100004"
                             b"c0000201")

        # This creates an A record for mail.example.com
        zone = self.create_zone()
        self.create_recordset(zone, 'A')

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_TXT(self):
        # query is for text.example.com. IN TXT
        payload = "d2f5012000010000000000010474657874076578616d706c6503636f6d00001000010000291000000000000000"  # noqa

        # expected_response is NOERROR.  The other fields are
        # id 54005
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA RD
        # edns 0
        # payload 8192
        # ;QUESTION
        # text.example.com. IN TXT
        # ;ANSWER
        # text.example.com. 3600 IN TXT "footxtdata"
        # ;AUTHORITY
        # ;ADDITIONAL

        expected_response = b"d2f5850000010001000000010474657874076578616d706c6503636f6d0000100001c00c0010000100000e10000b0a666f6f747874646174610000292000000000000000"  # noqa

        # This creates an TXT record for mail.example.com
        zone = self.create_zone()
        self.create_recordset(zone, 'TXT')

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()
        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_TXT_quoted_strings(self):
        # query is for text.example.com. IN TXT
        payload = "d2f5012000010000000000010474657874076578616d706c6503636f6d00001000010000291000000000000000"  # noqa

        expected_response = b"d2f5850000010001000000010474657874076578616d706c6503636f6d0000100001c00c0010000100000e10000d03666f6f0362617204626c61680000292000000000000000"  # noqa
        # expected_response is NOERROR.  The other fields are
        # response: id 54005
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA RD
        # edns 0
        # payload 8192
        # ;QUESTION
        # text.example.com. IN TXT
        # ;ANSWER
        # text.example.com. 3600 IN TXT "foo" "bar" "blah"
        # ;AUTHORITY
        # ;ADDITIONAL

        zone = self.create_zone()
        recordset = self.create_recordset(zone, recordset_type='TXT',
                                          records=[])
        values = {'data': '"foo" "bar" "blah"'}
        self.storage.create_record(
            self.admin_context, zone['id'], recordset['id'],
            objects.Record.from_dict(values))

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()
        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_MX(self):
        # query is for mail.example.com. IN MX
        payload = ("271701000001000000000000046d61696c076578616d706c6503636f6d"
                   "00000f0001")

        # expected_response is NOERROR.  The other fields are
        # id 10007
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA RD
        # ;QUESTION
        # mail.example.com. IN MX
        # ;ANSWER
        # mail.example.com. 3600 IN MX 5 mail.example.org.
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"271785000001000100000000046d61696c076578616d70"
                             b"6c6503636f6d00000f0001c00c000f000100000e100014"
                             b"0005046d61696c076578616d706c65036f726700")

        # This creates an MX record for mail.example.com
        zone = self.create_zone()
        self.create_recordset(zone, 'MX')

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_AXFR(self):
        # Query is for example.com. IN AXFR
        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags AD
        # edns 0
        # payload 4096
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        payload = ("49c300200001000000000001076578616d706c6503636f6d0000fc0001"
                   "0000291000000000000000")

        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # example.com. 3600 IN SOA ns1.example.org. example.example.com.
        # -> 1427899961 3600 600 86400 3600
        # mail.example.com. 3600 IN A 192.0.2.1
        # example.com. 3600 IN NS ns1.example.org.
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (
            b"49c384000001000400000000076578616d706c6503636f6d0000fc0001c0"
            b"0c0006000100000e10002f036e7331076578616d706c65036f7267000765786"
            b"16d706c65c00c551c063900000e10000002580001518000000e10c00c000200"
            b"0100000e100002c029046d61696cc00c0001000100000e100004c0000201c00"
            b"c0006000100000e100018c029c03a551c063900000e10000002580001518000"
            b"000e10")

        zone = objects.Zone.from_dict({
            'name': 'example.com.',
            'ttl': 3600,
            'serial': 1427899961,
            'email': 'example@example.com',
        })

        def _find_recordsets_axfr(context, criterion):
            if criterion['type'] == 'SOA':
                return [['UUID1', 'SOA', '3600', 'example.com.',
                         'ns1.example.org. example.example.com. 1427899961 '
                         '3600 600 86400 3600', 'ACTION']]

            elif criterion['type'] == '!SOA':
                return [
                    ['UUID2', 'NS', '3600', 'example.com.', 'ns1.example.org.',
                     'ACTION'],
                    ['UUID3', 'A', '3600', 'mail.example.com.', '192.0.2.1',
                     'ACTION'],
                ]

        with mock.patch.object(self.storage, 'find_zone',
                               return_value=zone):
            with mock.patch.object(self.storage, 'find_recordsets_axfr',
                                   side_effect=_find_recordsets_axfr):
                request = dns.message.from_wire(binascii.a2b_hex(payload))
                request.environ = {'addr': self.addr, 'context': self.context}

                response = next(self.handler(request)).get_wire()

                self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_AXFR_catalog(self):
        # Query is for example.com. IN AXFR
        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags AD
        # edns 0
        # payload 4096
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        payload = ("49c300200001000000000001076578616d706c6503636f6d0000fc0001"
                   "00fc000100000000000000")

        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # example.com. 3600 IN SOA ns1.example.org. example.example.com.
        # -> 1427899961 3600 600 86400 3600
        # mail.example.com. 3600 IN A 192.0.2.1
        # example.com. 3600 IN NS ns1.example.org.
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (
            b"49c384000001000500000000076578616d706c6503636f6d0000fc0001036361"
            b"74c00c0006000100000e10002707696e76616c696400076578616d706c65c00c"
            b"00000001000002580001518000000e1000000e10c01d0002000100000e100002"
            b"c02d0776657273696f6ec01d0010000100000e1000020132053138383833057a"
            b"6f6e6573c01d000c000100000e1000040161c00cc01d0006000100000e100018"
            b"c02dc03600000001000002580001518000000e1000000e10"
        )

        cat_zone = objects.Zone.from_dict({
            'name': 'cat.example.com.',
            'ttl': 3600,
            'serial': 1427899961,
            'email': 'example@example.com',
            'type': 'CATALOG'
        })

        pool = objects.Pool.from_dict({
            'name': 'POOL',

        })

        def _get_catalog_zone_records(context, pool):
            records = []

            soa_record = objects.RecordList()
            soa_record.append(
                objects.Record(
                    data='invalid. '
                         'example.example.com. '
                         '1 '
                         '600 '
                         '86400 '
                         '3600 '
                         '3600'
                )
            )
            soa = objects.RecordSet(
                name='cat.example.com.',
                type='SOA',
                records=soa_record
            )
            records.append(soa)

            ns_record = objects.RecordList()
            ns_record.append(objects.Record(data='invalid.'))
            records.append(
                objects.RecordSet(
                    name='cat.example.com.',
                    type='NS',
                    records=ns_record
                )
            )

            txt_record = objects.RecordList()
            txt_record.append(objects.Record(data='2'))
            records.append(
                objects.RecordSet(
                    name='version.cat.example.com.',
                    type='TXT',
                    records=txt_record
                )
            )

            ptr_record = objects.RecordList()
            ptr_record.append(
                objects.Record(
                    data='a.example.com.'
                )
            )
            records.append(
                objects.RecordSet(
                    name='18883.zones.cat.example.com.',
                    type='PTR',
                    records=ptr_record
                )
            )

            records.append(soa)

            return records

        with mock.patch.object(self.storage, 'find_zone',
                               return_value=cat_zone):
            with mock.patch.object(self.storage, 'find_pool',
                                   return_value=pool):
                with mock.patch.object(
                        self.storage, 'get_catalog_zone_records',
                        side_effect=_get_catalog_zone_records):
                    request = dns.message.from_wire(binascii.a2b_hex(payload))
                    request.environ = {'addr': self.addr,
                                       'context': self.context}

                    response = next(self.handler(request)).get_wire()

                    self.assertEqual(
                        expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_AXFR_multiple_messages(self):
        # Query is for example.com. IN AXFR
        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags AD
        # edns 0
        # payload 4096
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        payload = ("49c300200001000000000001076578616d706c6503636f6d0000fc0001"
                   "0000291000000000000000")

        expected_response = [
            (b"49c384000001000300000000076578616d706c6503636f6d0000fc0001c00c"
             b"0006000100000e10002f036e7331076578616d706c65036f726700076578616"
             b"d706c65c00c551c063900000e10000002580001518000000e10c00c0002000"
             b"100000e100002c029046d61696cc00c0001000100000e100004c0000201"),

            (b"49c384000001000100000000076578616d706c6503636f6d0000fc0001c00c"
             b"0006000100000e10002f036e7331076578616d706c65036f72670007657861"
             b"6d706c65c00c551c063900000e10000002580001518000000e10"),
        ]

        # Set the max-message-size to 128
        self.config(max_message_size=128, group='service:mdns')

        zone = objects.Zone.from_dict({
            'name': 'example.com.',
            'ttl': 3600,
            'serial': 1427899961,
            'email': 'example@example.com',
        })

        def _find_recordsets_axfr(context, criterion):
            if criterion['type'] == 'SOA':
                return [['UUID1', 'SOA', '3600', 'example.com.',
                         'ns1.example.org. example.example.com. 1427899961 '
                         '3600 600 86400 3600', 'ACTION']]

            elif criterion['type'] == '!SOA':
                return [
                    ['UUID2', 'NS', '3600', 'example.com.', 'ns1.example.org.',
                     'ACTION'],
                    ['UUID3', 'A', '3600', 'mail.example.com.', '192.0.2.1',
                     'ACTION'],
                ]

        with mock.patch.object(self.storage, 'find_zone',
                               return_value=zone):
            with mock.patch.object(self.storage, 'find_recordsets_axfr',
                                   side_effect=_find_recordsets_axfr):
                request = dns.message.from_wire(binascii.a2b_hex(payload))
                request.environ = {'addr': self.addr, 'context': self.context}

                response_generator = self.handler(request)

                # Validate the first response
                response_one = next(response_generator).get_wire()
                self.assertEqual(
                    expected_response[0], binascii.b2a_hex(response_one))

                # Validate the second response
                response_two = next(response_generator).get_wire()
                self.assertEqual(
                    expected_response[1], binascii.b2a_hex(response_two))

    @mock.patch.object(dns.renderer.Renderer, 'add_multi_tsig')
    def test_dispatch_opcode_query_AXFR_multiple_messages_with_tsig(
            self, mock_multi_tsig):
        # Query is for example.com. IN AXFR
        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags AD
        # edns 0
        # payload 4096
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        payload = ("49c300200001000000000001076578616d706c6503636f6d0000fc0001"
                   "0000291000000000000000")

        expected_response = [
            (b"49c384000001000300000000076578616d706c6503636f6d0000fc0001c00c"
             b"0006000100000e10002f036e7331076578616d706c65036f726700076578616"
             b"d706c65c00c551c063900000e10000002580001518000000e10c00c0002000"
             b"100000e100002c029046d61696cc00c0001000100000e100004c0000201"),

            (b"49c384000001000100000000076578616d706c6503636f6d0000fc0001c00c"
             b"0006000100000e10002f036e7331076578616d706c65036f72670007657861"
             b"6d706c65c00c551c063900000e10000002580001518000000e10"),
        ]

        # Set the max-message-size to 363
        self.config(max_message_size=363, group='service:mdns')

        zone = objects.Zone.from_dict({
            'name': 'example.com.',
            'ttl': 3600,
            'serial': 1427899961,
            'email': 'example@example.com',
        })

        def _find_recordsets_axfr(context, criterion):
            if criterion['type'] == 'SOA':
                return [['UUID1', 'SOA', '3600', 'example.com.',
                         'ns1.example.org. example.example.com. 1427899961 '
                         '3600 600 86400 3600', 'ACTION']]

            elif criterion['type'] == '!SOA':
                return [
                    ['UUID2', 'NS', '3600', 'example.com.', 'ns1.example.org.',
                     'ACTION'],
                    ['UUID3', 'A', '3600', 'mail.example.com.', '192.0.2.1',
                     'ACTION'],
                ]

        with mock.patch.object(self.storage, 'find_zone',
                               return_value=zone):
            with mock.patch.object(self.storage, 'find_recordsets_axfr',
                                   side_effect=_find_recordsets_axfr):
                request = dns.message.from_wire(binascii.a2b_hex(payload))
                request.environ = {'addr': self.addr, 'context': self.context}

                request.use_tsig(dns.tsigkeyring.from_text(
                    {'test-key-two': 'AnotherSecretKey'})
                )

                args = [request.keyname, request.keyring.secret,
                        300, request.id, request.tsig_error,
                        b'', request.mac, request.keyalgorithm]
                response_generator = self.handler(request)
                # Validate the first response
                response_one = next(response_generator).get_wire()
                mock_multi_tsig.assert_called_with(None, *args)
                self.assertEqual(
                    expected_response[0], binascii.b2a_hex(response_one))

                # Validate the second response
                response_two = next(response_generator).get_wire()
                first_msg_ctx = mock_multi_tsig.return_value
                mock_multi_tsig.assert_called_with(first_msg_ctx, *args)
                self.assertEqual(
                    expected_response[1], binascii.b2a_hex(response_two))

    def test_dispatch_opcode_query_AXFR_rrset_over_max_size(self):
        # Query is for example.com. IN AXFR
        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags AD
        # edns 0
        # payload 4096
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        payload = ("49c300200001000000000001076578616d706c6503636f6d0000fc0001"
                   "0000291000000000000000")

        expected_response = [
            # Initial SOA
            (b"49c384000001000100000000076578616d706c6503636f6d0000fc0001c00c0"
             b"006000100000e10002f036e7331076578616d706c65036f726700076578616d"
             b"706c65c00c551c063900000e10000002580001518000000e10"),

            # First NS record
            (b"49c384000001000100000000076578616d706c6503636f6d0000fc0001c00c0"
             b"002000100000e1000413f616161616161616161616161616161616161616161"
             b"616161616161616161616161616161616161616161616161616161616161616"
             b"16161616161616161616100"),

            # Second NS Record and SOA trailer
            (b"49c384000001000200000000076578616d706c6503636f6d0000fc0001c00c0"
             b"002000100000e10000c0a6262626262626262626200c00c0006000100000e10"
             b"002f036e7331076578616d706c65036f726700076578616d706c65c00c551c0"
             b"63900000e10000002580001518000000e10"),
        ]

        # Set the max-message-size to 128
        self.config(max_message_size=128, group='service:mdns')

        zone = objects.Zone.from_dict({
            'name': 'example.com.',
            'ttl': 3600,
            'serial': 1427899961,
            'email': 'example@example.com',
        })

        def _find_recordsets_axfr(context, criterion):
            if criterion['type'] == 'SOA':
                return [['UUID1', 'SOA', '3600', 'example.com.',
                         'ns1.example.org. example.example.com. 1427899961 '
                         '3600 600 86400 3600', 'ACTION']]

            elif criterion['type'] == '!SOA':
                return [
                    ['UUID2', 'NS', '3600', 'example.com.', 'a' * 63 + '.',
                     'ACTION'],
                    ['UUID2', 'NS', '3600', 'example.com.', 'b' * 10 + '.',
                     'ACTION'],
                ]

        with mock.patch.object(self.storage, 'find_zone',
                               return_value=zone):
            with mock.patch.object(self.storage, 'find_recordsets_axfr',
                                   side_effect=_find_recordsets_axfr):
                request = dns.message.from_wire(binascii.a2b_hex(payload))
                request.environ = {'addr': self.addr, 'context': self.context}

                response_generator = self.handler(request)

                # Validate the first response
                response_one = next(response_generator).get_wire()
                self.assertEqual(
                    expected_response[0], binascii.b2a_hex(response_one))

                # Validate the second response
                response_two = next(response_generator).get_wire()
                self.assertEqual(
                    expected_response[1], binascii.b2a_hex(response_two))

                # Validate the third response
                response_three = next(response_generator).get_wire()
                self.assertEqual(
                    expected_response[2], binascii.b2a_hex(response_three))

                # Ensure a StopIteration is raised after the final response.
                with testtools.ExpectedException(StopIteration):
                    next(response_generator)

    def test_dispatch_opcode_query_AXFR_rr_over_max_size(self):
        # Query is for example.com. IN AXFR
        # id 18883
        # opcode QUERY
        # rcode NOERROR
        # flags AD
        # edns 0
        # payload 4096
        # ;QUESTION
        # example.com. IN AXFR
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        payload = ("49c300200001000000000001076578616d706c6503636f6d0000fc0001"
                   "0000291000000000000000")

        expected_response = [
            # Initial SOA
            (b"49c384000001000100000000076578616d706c6503636f6d0000fc0001c00c0"
             b"006000100000e10002f036e7331076578616d706c65036f726700076578616d"
             b"706c65c00c551c063900000e10000002580001518000000e10"),

            # SRVFAIL
            (""),
        ]

        # Set the max-message-size to 128
        self.config(max_message_size=128, group='service:mdns')

        zone = objects.Zone.from_dict({
            'name': 'example.com.',
            'ttl': 3600,
            'serial': 1427899961,
            'email': 'example@example.com',
        })

        def _find_recordsets_axfr(context, criterion):
            if criterion['type'] == 'SOA':
                return [['UUID1', 'SOA', '3600', 'example.com.',
                         'ns1.example.org. example.example.com. 1427899961 '
                         '3600 600 86400 3600', 'ACTION']]

            elif criterion['type'] == '!SOA':
                return [
                    ['UUID2', 'NS', '3600', 'example.com.',
                     'a' * 63 + '.' + 'a' * 63 + '.', 'ACTION'],
                ]

        with mock.patch.object(self.storage, 'find_zone',
                               return_value=zone):
            with mock.patch.object(self.storage, 'find_recordsets_axfr',
                                   side_effect=_find_recordsets_axfr):
                request = dns.message.from_wire(binascii.a2b_hex(payload))
                request.environ = {'addr': self.addr, 'context': self.context}

                response_generator = self.handler(request)

                # Validate the first response
                response_one = next(response_generator).get_wire()
                self.assertEqual(
                    expected_response[0], binascii.b2a_hex(response_one))

                # Validate the second response is a SERVFAIL
                response_two = next(response_generator)
                self.assertEqual(
                    dns.rcode.SERVFAIL, response_two.rcode())

                # Ensure a StopIteration is raised after the final response.
                with testtools.ExpectedException(StopIteration):
                    next(response_generator)

    def test_dispatch_opcode_query_nonexistent_recordtype(self):
        # query is for mail.example.com. IN CNAME
        payload = ("271801000001000000000000046d61696c076578616d706c6503636f6d"
                   "0000050001")

        # expected_response is REFUSED.  The other fields are
        # id 10008
        # opcode QUERY
        # rcode REFUSED
        # flags QR RD
        # ;QUESTION
        # mail.example.com. IN CNAME
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"271881050001000000000000046d61696c076578616d70"
                             b"6c6503636f6d0000050001")

        # This creates an MX record for mail.example.com
        # But we query for a CNAME record
        zone = self.create_zone()
        self.create_recordset(zone, 'MX')

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_unsupported_recordtype(self):
        # query is for example.com. IN DNAME
        payload = "271901000001000000000000076578616d706c6503636f6d0000270001"

        # expected_response is REFUSED.  The other fields are
        # id 10009
        # opcode QUERY
        # rcode REFUSED
        # flags QR RD
        # ;QUESTION
        # example.com. IN DNAME
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"271981050001000000000000076578616d706c6503636f"
                             b"6d0000270001")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {'addr': self.addr, 'context': self.context}
        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_tsig_scope_pool(self):
        # Create a zone/recordset/record to query
        zone = self.create_zone(name='example.com.')
        self.create_recordset(
            zone, name='example.com.', recordset_type='A'
        )

        # DNS packet with QUERY opcode for A example.com.
        payload = ("c28901200001000000000001076578616d706c6503636f6d0000010001"
                   "0000291000000000000000")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {
            'addr': self.addr,
            'context': self.context,
            'tsigkey': self.tsigkey_pool_default,
        }

        # Ensure the Query, with the correct pool's TSIG, gives a NOERROR.
        # id 49801
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA RD
        # edns 0
        # payload 8192
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # example.com. 3600 IN A 192.0.2.5
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (
            b'c28985000001000100000001076578616d706c6503636f6d0000010001c00c00'
            b'01000100000e100004c00002010000292000000000000000'
        )

        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

        # Ensure the Query, with the incorrect pool's TSIG, gives a REFUSED
        request.environ['tsigkey'] = self.tsigkey_pool_unknown

        # id 49801
        # opcode QUERY
        # rcode REFUSED
        # flags QR RD
        # edns 0
        # payload 8192
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"c28981050001000000000001076578616d706c6503636f"
                             b"6d00000100010000292000000000000000")

        response = next(self.handler(request)).to_wire()
        self.assertEqual(expected_response, binascii.b2a_hex(response))

    def test_dispatch_opcode_query_tsig_scope_zone(self):
        # Create a zone/recordset/record to query
        zone = self.create_zone(name='example.com.')
        records = [
            objects.Record.from_dict({'data': '192.0.2.5'}),
        ]
        self.create_recordset(
            zone, name='example.com.', recordset_type='A', records=records
        )

        # Create a TSIG Key Matching the zone
        tsigkey_zone_known = self.create_tsigkey(
            name='known-zone',
            scope='ZONE',
            resource_id=zone.id)

        # DNS packet with QUERY opcode for A example.com.
        payload = ("c28901200001000000000001076578616d706c6503636f6d0000010001"
                   "0000291000000000000000")

        request = dns.message.from_wire(binascii.a2b_hex(payload))
        request.environ = {
            'addr': self.addr,
            'context': self.context,
            'tsigkey': tsigkey_zone_known,
        }

        # Ensure the Query, with the correct zone's TSIG, gives a NOERROR.
        # id 49801
        # opcode QUERY
        # rcode NOERROR
        # flags QR AA RD
        # edns 0
        # payload 8192
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # example.com. 3600 IN A 192.0.2.5
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"c28985000001000100000001076578616d706c6503636f"
                             b"6d0000010001c00c0001000100000e100004c000020500"
                             b"00292000000000000000")

        response = next(self.handler(request)).to_wire()

        self.assertEqual(expected_response, binascii.b2a_hex(response))

        # Ensure the Query, with the incorrect zone's TSIG, gives a REFUSED
        request.environ['tsigkey'] = self.tsigkey_zone_unknown

        # id 49801
        # opcode QUERY
        # rcode REFUSED
        # flags QR RD
        # edns 0
        # payload 8192
        # ;QUESTION
        # example.com. IN A
        # ;ANSWER
        # ;AUTHORITY
        # ;ADDITIONAL
        expected_response = (b"c28981050001000000000001076578616d706c6503636f"
                             b"6d00000100010000292000000000000000")

        response = next(self.handler(request)).to_wire()
        self.assertEqual(expected_response, binascii.b2a_hex(response))



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg

HEARTBEAT_GROUP = cfg.OptGroup(
    name='heartbeat_emitter',
    title="Configuration for heartbeat_emitter"
)

HEARTBEAT_OPTS = [
    cfg.FloatOpt('heartbeat_interval',
                 default=10.0,
                 help='Number of seconds between heartbeats for reporting '
                      'state'),
    cfg.StrOpt('emitter_type', default="rpc", help="Emitter to use"),
]


def register_opts(conf):
    conf.register_group(HEARTBEAT_GROUP)
    conf.register_opts(HEARTBEAT_OPTS, group=HEARTBEAT_GROUP)


def list_opts():
    return {
        HEARTBEAT_GROUP: HEARTBEAT_OPTS
    }



# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2022 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""liberty

Revision ID: c9f427f7180a
Revises:
Create Date: 2022-07-28 23:06:40.731452

"""
from alembic import op
from oslo_utils import timeutils
import sqlalchemy as sa

import designate.conf
from designate.conf import central
from designate.storage.sqlalchemy.alembic import legacy_utils
from designate.storage.sqlalchemy.types import UUID

# revision identifiers, used by Alembic.
revision = 'c9f427f7180a'
down_revision = None
branch_labels = None
depends_on = None

# Equivalent to legacy sqlalchemy-migrate revision 070_liberty

CONF = designate.conf.CONF
central.register_opts(CONF)

ACTIONS = ('CREATE', 'DELETE', 'UPDATE', 'NONE')
POOL_PROVISIONERS = ('UNMANAGED',)
RECORD_TYPES = ('A', 'AAAA', 'CNAME', 'MX', 'SRV', 'TXT', 'SPF', 'NS', 'PTR',
                'SSHFP', 'SOA')
RESOURCE_STATUSES = ('ACTIVE', 'PENDING', 'DELETED', 'ERROR')
TASK_STATUSES = ('ACTIVE', 'PENDING', 'DELETED', 'ERROR', 'COMPLETE')
TSIG_ALGORITHMS = ('hmac-md5', 'hmac-sha1', 'hmac-sha224', 'hmac-sha256',
                   'hmac-sha384', 'hmac-sha512')
TSIG_SCOPES = ('POOL', 'ZONE')
ZONE_ATTRIBUTE_KEYS = ('master',)
ZONE_TASK_TYPES = ('IMPORT', 'EXPORT')
ZONE_TYPES = ('PRIMARY', 'SECONDARY')


def upgrade() -> None:
    # Check if the equivalent legacy migration has already run
    if not legacy_utils.is_migration_needed(70):
        return

    metadata = sa.MetaData()

    pools_table = op.create_table(
        'pools', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('name', sa.String(50), nullable=False),
        sa.Column('description', sa.Unicode(160), nullable=True),
        sa.Column('tenant_id', sa.String(36), nullable=True),
        sa.Column('provisioner', sa.Enum(name='pool_provisioner',
                                         *POOL_PROVISIONERS),
                  nullable=False, server_default='UNMANAGED'),
        sa.UniqueConstraint('name', name='unique_pool_name'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.bulk_insert(
        pools_table,
        [{'id': CONF['service:central'].default_pool_id,
          'name': 'default',
          'version': 1}])

    op.create_table(
        'pool_ns_records', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('pool_id', UUID, nullable=False),
        sa.Column('priority', sa.Integer, nullable=False),
        sa.Column('hostname', sa.String(255), nullable=False),
        sa.ForeignKeyConstraint(['pool_id'], ['pools.id'], ondelete='CASCADE'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'pool_attributes', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('key', sa.String(255), nullable=False),
        sa.Column('value', sa.String(255), nullable=False),
        sa.Column('pool_id', UUID, nullable=False),
        sa.UniqueConstraint('pool_id', 'key', 'value',
                            name='unique_pool_attribute'),
        sa.ForeignKeyConstraint(['pool_id'], ['pools.id'], ondelete='CASCADE'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'domains', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('version', sa.Integer, nullable=False),
        sa.Column('tenant_id', sa.String(36), default=None, nullable=True),
        sa.Column('name', sa.String(255), nullable=False),
        sa.Column('email', sa.String(255), nullable=False),
        sa.Column('ttl', sa.Integer, default=lambda: CONF.default_ttl,
                  nullable=False),
        sa.Column('refresh', sa.Integer, nullable=False),
        sa.Column('retry', sa.Integer, nullable=False),
        sa.Column('expire', sa.Integer, nullable=False),
        sa.Column('minimum', sa.Integer, nullable=False),
        sa.Column('parent_domain_id', UUID, default=None, nullable=True),
        sa.Column('serial', sa.Integer, nullable=False, server_default='1'),
        sa.Column('deleted', sa.CHAR(32), nullable=False, default='0',
                  server_default='0'),
        sa.Column('deleted_at', sa.DateTime, nullable=True, default=None),
        sa.Column('description', sa.Unicode(160), nullable=True),
        sa.Column('status', sa.Enum(name='domains_resource_statuses',
                                    *RESOURCE_STATUSES),
                  nullable=False, server_default='PENDING', default='PENDING'),
        sa.Column('action', sa.Enum(name='domain_actions', *ACTIONS),
                  default='CREATE', server_default='CREATE', nullable=False),
        sa.Column('pool_id', UUID, default=None, nullable=True),
        sa.Column('reverse_name', sa.String(255), nullable=False,
                  server_default=''),
        sa.Column("type", sa.Enum(name='type', *ZONE_TYPES),
                  server_default='PRIMARY', default='PRIMARY'),
        sa.Column('transferred_at', sa.DateTime, default=None),
        sa.Column('shard', sa.SmallInteger, nullable=False),
        sa.UniqueConstraint('name', 'deleted', 'pool_id',
                            name='unique_domain_name'),
        sa.ForeignKeyConstraint(['parent_domain_id'],
                                ['domains.id'],
                                ondelete='SET NULL'),
        sa.Index('zone_deleted', 'deleted'),
        sa.Index('zone_tenant_deleted', 'tenant_id', 'deleted'),
        sa.Index('reverse_name_deleted', 'reverse_name', 'deleted'),
        sa.Index('zone_created_at', 'created_at'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'domain_attributes', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('key', sa.Enum(name='key', *ZONE_ATTRIBUTE_KEYS)),
        sa.Column('value', sa.String(255), nullable=False),
        sa.Column('domain_id', UUID, nullable=False),
        sa.UniqueConstraint('key', 'value', 'domain_id',
                            name='unique_attributes'),
        sa.ForeignKeyConstraint(['domain_id'], ['domains.id'],
                                ondelete='CASCADE'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )
    op.create_table(
        'recordsets', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('domain_shard', sa.SmallInteger, nullable=False),
        sa.Column('tenant_id', sa.String(36), default=None, nullable=True),
        sa.Column('domain_id', UUID, nullable=False),
        sa.Column('name', sa.String(255), nullable=False),
        sa.Column('type', sa.Enum(name='record_types', *RECORD_TYPES),
                  nullable=False),
        sa.Column('ttl', sa.Integer, default=None, nullable=True),
        sa.Column('description', sa.Unicode(160), nullable=True),
        sa.Column('reverse_name', sa.String(255), nullable=False,
                  server_default=''),
        sa.UniqueConstraint('domain_id', 'name', 'type',
                            name='unique_recordset'),
        sa.ForeignKeyConstraint(['domain_id'], ['domains.id'],
                                ondelete='CASCADE'),
        sa.Index('rrset_type_domainid', 'type', 'domain_id'),
        sa.Index('recordset_type_name', 'type', 'name'),
        sa.Index('reverse_name_dom_id', 'reverse_name', 'domain_id'),
        sa.Index('recordset_created_at', 'created_at'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'records', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('data', sa.Text, nullable=False),
        sa.Column('domain_id', UUID, nullable=False),
        sa.Column('managed', sa.Boolean, default=False),
        sa.Column('managed_resource_type', sa.Unicode(50), default=None,
                  nullable=True),
        sa.Column('managed_resource_id', UUID, default=None, nullable=True),
        sa.Column('managed_plugin_type', sa.Unicode(50), default=None,
                  nullable=True),
        sa.Column('managed_plugin_name', sa.Unicode(50), default=None,
                  nullable=True),
        sa.Column('hash', sa.String(32), nullable=False),
        sa.Column('description', sa.Unicode(160), nullable=True),
        sa.Column('status', sa.Enum(name='record_resource_statuses',
                                    *RESOURCE_STATUSES),
                  server_default='PENDING', default='PENDING', nullable=False),
        sa.Column('tenant_id', sa.String(36), default=None, nullable=True),
        sa.Column('recordset_id', UUID, nullable=False),
        sa.Column('managed_tenant_id', sa.Unicode(36), default=None,
                  nullable=True),
        sa.Column('managed_resource_region', sa.Unicode(100), default=None,
                  nullable=True),
        sa.Column('managed_extra', sa.Unicode(100), default=None,
                  nullable=True),
        sa.Column('action', sa.Enum(name='record_actions', *ACTIONS),
                  default='CREATE', server_default='CREATE', nullable=False),
        sa.Column('serial', sa.Integer, server_default='1', nullable=False),
        sa.Column('domain_shard', sa.SmallInteger, nullable=False),
        sa.UniqueConstraint('hash', name='unique_record'),
        sa.ForeignKeyConstraint(['domain_id'], ['domains.id'],
                                ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['recordset_id'], ['recordsets.id'],
                                ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['domain_id'], ['domains.id'],
                                ondelete='CASCADE',
                                name='fkey_records_domain_id'),
        sa.ForeignKeyConstraint(['recordset_id'], ['recordsets.id'],
                                ondelete='CASCADE',
                                name='fkey_records_recordset_id'),
        sa.Index('records_tenant', 'tenant_id'),
        sa.Index('record_created_at', 'created_at'),
        sa.Index('update_status_index', 'status', 'domain_id', 'tenant_id',
                 'created_at', 'serial'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'quotas', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('created_at', sa.DateTime,
                  default=lambda: timeutils.utcnow()),
        sa.Column('updated_at', sa.DateTime,
                  onupdate=lambda: timeutils.utcnow()),
        sa.Column('tenant_id', sa.String(36), nullable=False),
        sa.Column('resource', sa.String(32), nullable=False),
        sa.Column('hard_limit', sa.Integer, nullable=False),
        sa.UniqueConstraint('tenant_id', 'resource', name='unique_quota'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'tsigkeys', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('name', sa.String(255), nullable=False),
        sa.Column('algorithm',
                  sa.Enum(name='tsig_algorithms', *TSIG_ALGORITHMS),
                  nullable=False),
        sa.Column('secret', sa.String(255), nullable=False),
        sa.Column('scope', sa.Enum(name='tsig_scopes', *TSIG_SCOPES),
                  nullable=False, server_default='POOL'),
        sa.Column('resource_id', UUID, nullable=False),
        sa.UniqueConstraint('name', name='unique_tsigkey_name'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'tlds', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('created_at', sa.DateTime,
                  default=lambda: timeutils.utcnow()),
        sa.Column('updated_at', sa.DateTime,
                  onupdate=lambda: timeutils.utcnow()),
        sa.Column('name', sa.String(255), nullable=False),
        sa.Column('description', sa.Unicode(160), nullable=True),
        sa.UniqueConstraint('name', name='unique_tld_name'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'zone_transfer_requests', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('domain_id', UUID, nullable=False),
        sa.Column("key", sa.String(255), nullable=False),
        sa.Column("description", sa.String(255)),
        sa.Column("tenant_id", sa.String(36), default=None, nullable=False),
        sa.Column("target_tenant_id", sa.String(36), default=None,
                  nullable=True),
        sa.Column("status",
                  sa.Enum(name='zone_transfer_requests_resource_statuses',
                          *TASK_STATUSES),
                  nullable=False, server_default='ACTIVE', default='ACTIVE'),
        sa.ForeignKeyConstraint(['domain_id'], ['domains.id'],
                                ondelete='CASCADE'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'zone_transfer_accepts', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('domain_id', UUID, nullable=False),
        sa.Column('zone_transfer_request_id', UUID, nullable=False),
        sa.Column("tenant_id", sa.String(36), default=None, nullable=False),
        sa.Column("status",
                  sa.Enum(name='zone_transfer_accepts_resource_statuses',
                          *TASK_STATUSES),
                  nullable=False, server_default='ACTIVE', default='ACTIVE'),
        sa.ForeignKeyConstraint(['domain_id'], ['domains.id'],
                                ondelete='CASCADE'),
        sa.ForeignKeyConstraint(['zone_transfer_request_id'],
                                ['zone_transfer_requests.id'],
                                ondelete='CASCADE'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'zone_tasks', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('tenant_id', sa.String(36), default=None, nullable=True),
        sa.Column('domain_id', UUID, nullable=True),
        sa.Column('task_type', sa.Enum(name='task_types', *ZONE_TASK_TYPES),
                  nullable=True),
        sa.Column('message', sa.String(160), nullable=True),
        sa.Column('status', sa.Enum(name='zone_tasks_resource_statuses',
                                    *TASK_STATUSES),
                  nullable=False, server_default='ACTIVE', default='ACTIVE'),
        sa.Column('location', sa.String(160), nullable=True),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )

    op.create_table(
        'blacklists', metadata,
        sa.Column('id', UUID, primary_key=True),
        sa.Column('version', sa.Integer, default=1, nullable=False),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('created_at', sa.DateTime),
        sa.Column('pattern', sa.String(255), nullable=False),
        sa.Column('description', sa.Unicode(160), nullable=True),
        sa.UniqueConstraint('pattern', name='pattern'),
        mysql_engine='InnoDB',
        mysql_charset='utf8',
    )



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging

from designate import exceptions
from designate import objects
from designate.quota import base
from designate import storage
from designate.storage import transaction


LOG = logging.getLogger(__name__)


class StorageQuota(base.Quota):
    __plugin_name__ = 'storage'

    def __init__(self):
        super().__init__()
        self.storage = storage.get_storage()

    def _get_quotas(self, context, tenant_id):
        """Internal Get Quotas used by get_quotas"""
        quotas = self.storage.find_quotas(context, {
            'tenant_id': tenant_id,
        })

        return {q['resource']: q['hard_limit'] for q in quotas}

    def get_quota(self, context, tenant_id, resource):
        """Get Quota"""
        context = context.deepcopy()
        context.all_tenants = True

        quota = self.storage.find_quota(context, {
            'tenant_id': tenant_id,
            'resource': resource,
        })

        return {
            resource: quota['hard_limit']
        }

    @transaction
    def set_quota(self, context, tenant_id, resource, hard_limit):
        """Set Quota"""
        context = context.deepcopy()
        context.all_tenants = True

        if resource not in list(self.get_default_quotas(context).keys()):
            raise exceptions.QuotaResourceUnknown(
                f'{resource} is not a valid quota resource'
            )

        try:
            self._create_quota(context, tenant_id, resource, hard_limit)
        except exceptions.Duplicate:
            quota = self.storage.find_quota(context, {
                'tenant_id': tenant_id,
                'resource': resource,
            })
            self._update_quota(context, quota, hard_limit)

        return {resource: hard_limit}

    def _create_quota(self, context, project_id, resource, hard_limit):
        quota = objects.Quota(
            tenant_id=project_id, resource=resource, hard_limit=hard_limit
        )
        self.storage.create_quota(context, quota)

    def _update_quota(self, context, quota, hard_limit):
        quota.hard_limit = hard_limit
        self.storage.update_quota(context, quota)

    @transaction
    def reset_quotas(self, context, tenant_id):
        """Reset Quotas"""
        context = context.deepcopy()
        context.all_tenants = True

        quotas = self.storage.find_quotas(context, {
            'tenant_id': tenant_id,
        })

        for quota in quotas:
            self.storage.delete_quota(context, quota['id'])



# Copyright 2016 Rackspace
#
# Author: Rahman Syed <rahman.syed@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import oslotest.base

from designate.exceptions import InvalidObject
from designate import objects

LOG = logging.getLogger(__name__)


class RRDataMXTest(oslotest.base.BaseTestCase):
    def test_parse_mx(self):
        mx_record = objects.MX()
        mx_record.from_string('0 mail.example.org.')

        self.assertEqual(0, mx_record.priority)
        self.assertEqual('mail.example.org.', mx_record.exchange)

    def test_validate_mx_signed_zero(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='MX',
            records=objects.RecordList(objects=[
                objects.Record(data='-0 mail.example.test.',
                               status='ACTIVE'),
            ])
        )

        self.assertRaises(InvalidObject, record_set.validate)

    def test_validate_mx_not_fqdn(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='MX',
            records=objects.RecordList(objects=[
                objects.Record(data='10 mail.example.test',
                               status='ACTIVE'),
            ])
        )

        self.assertRaises(InvalidObject, record_set.validate)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import errno
import socket
import struct
from unittest import mock

from oslo_config import fixture as cfg_fixture
from oslo_service import service
import oslotest.base

from designate.common import profiler
import designate.conf
from designate.mdns import handler
from designate import policy
from designate import rpc
from designate import service as designate_service
from designate.tests import base_fixtures
from designate import utils


CONF = designate.conf.CONF


@mock.patch.object(designate_service, '_launcher', None)
class BaseServiceTest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

    @mock.patch.object(service, 'launch')
    def test_serve(self, mock_service_launch):
        server = mock.Mock()
        designate_service.serve(server)

        mock_service_launch.assert_called_with(
            mock.ANY, server, workers=None, restart_method='mutate'
        )

    @mock.patch.object(service, 'launch', mock.Mock())
    def test_serve_twice(self):
        server = mock.Mock()
        designate_service.serve(server)
        self.assertRaisesRegex(
            RuntimeError,
            r'serve\(\) can only be called once',
            designate_service.serve, server
        )

    @mock.patch.object(rpc, 'cleanup')
    @mock.patch.object(service, 'launch')
    def test_wait(self, mock_service_launch, mock_rpc_cleanup):
        server = mock.Mock()

        designate_service.serve(server)
        designate_service.wait()

        mock_service_launch.assert_called_with(
            mock.ANY, server, workers=None, restart_method='mutate'
        )
        mock_rpc_cleanup.assert_called()

    @mock.patch.object(rpc, 'cleanup')
    @mock.patch.object(service, 'launch')
    def test_wait_keyboard_interrupt(self, mock_service_launch,
                                     mock_rpc_cleanup):
        server = mock.Mock()
        mock_launcher = mock.Mock()
        mock_service_launch.return_value = mock_launcher
        mock_launcher.wait.side_effect = [KeyboardInterrupt]

        designate_service.serve(server)
        designate_service.wait()

        mock_rpc_cleanup.assert_called()

        self.assertIn(
            'Caught KeyboardInterrupt, shutting down now',
            self.stdlog.logger.output
        )


@mock.patch.object(policy, 'init')
@mock.patch.object(rpc, 'init')
@mock.patch.object(rpc, 'initialized')
@mock.patch.object(profiler, 'setup_profiler')
class TestServiceInit(oslotest.base.BaseTestCase):
    def test_service_init(self, mock_setup_profiler, mock_rpc_initialized,
                          mock_rpc_init, mock_policy_init):
        mock_rpc_initialized.return_value = False

        service = designate_service.Service('test-service')

        mock_rpc_initialized.assert_called_once()
        mock_policy_init.assert_called_once()
        mock_rpc_init.assert_called_once()
        mock_setup_profiler.assert_called_once()

        self.assertEqual('test-service', service.name)

    def test_rpc_service_init(self, mock_setup_profiler, mock_rpc_initialized,
                              mock_rpc_init, mock_policy_init):
        mock_rpc_initialized.return_value = False

        service = designate_service.RPCService(
            'test-rpc-service', 'test-topic'
        )

        mock_rpc_initialized.assert_called_once()
        mock_policy_init.assert_called_once()
        mock_rpc_init.assert_called_once()
        mock_setup_profiler.assert_called_once()

        self.assertEqual([service], service.endpoints)
        self.assertEqual('test-topic', service.rpc_topic)
        self.assertEqual('test-rpc-service', service.name)


class TestRpcService(oslotest.base.BaseTestCase):
    @mock.patch.object(policy, 'init')
    @mock.patch.object(rpc, 'init')
    @mock.patch.object(rpc, 'initialized')
    @mock.patch.object(profiler, 'setup_profiler')
    def setUp(self, mock_setup_profiler, mock_rpc_initialized, mock_rpc_init,
              mock_policy_init):
        super().setUp()

        mock_rpc_initialized.return_value = False

        self.service = designate_service.RPCService(
            'test-rpc-service', 'test-topic'
        )

        mock_policy_init.assert_called_once()
        mock_rpc_initialized.assert_called_once()
        mock_rpc_init.assert_called_once()
        mock_setup_profiler.assert_called_once()

    @mock.patch.object(rpc, 'get_server')
    @mock.patch.object(rpc, 'get_notifier')
    def test_rpc_service_start(self, mock_rpc_get_server,
                               mock_rpc_get_notifier):
        self.assertIsNone(self.service.start())

        mock_rpc_get_server.assert_called_once()
        mock_rpc_get_notifier.assert_called_once()

        self.service.rpc_server.start.assert_called_once()

    @mock.patch.object(rpc, 'get_server')
    @mock.patch.object(rpc, 'get_notifier')
    def test_rpc_service_stop(self, mock_rpc_get_server,
                              mock_rpc_get_notifier):
        self.assertIsNone(self.service.start())

        mock_rpc_get_server.assert_called_once()
        mock_rpc_get_notifier.assert_called_once()

        self.assertIsNone(self.service.stop())

        self.service.rpc_server.stop.assert_called_once()

    def test_rpc_service_wait(self):
        self.assertIsNone(self.service.wait())


@mock.patch.object(policy, 'init', mock.Mock())
@mock.patch.object(rpc, 'init', mock.Mock())
@mock.patch.object(profiler, 'setup_profiler', mock.Mock())
class TestWSGIService(oslotest.base.BaseTestCase):
    @mock.patch('oslo_service.wsgi.Server')
    def test_service_start(self, mock_wsgi_server):
        mock_server = mock.Mock(name='server')
        mock_wsgi_server.return_value = mock_server
        listen = [
            ('192.0.2.1', '80'),
            ('192.0.2.2', '443'),
            ('192.0.2.2', '53')
        ]

        self.mock_app = mock.Mock()

        self.service = designate_service.WSGIService(
            self.mock_app, 'test-wsgi-service', listen
        )
        mock_wsgi_server.assert_called()
        self.assertEqual(3, mock_wsgi_server.call_count)

        self.assertIsNone(self.service.start())
        mock_server.start.assert_called()
        self.assertEqual(3, mock_server.start.call_count)

    @mock.patch('oslo_service.wsgi.Server')
    def test_service_stop(self, mock_wsgi_server):
        mock_server = mock.Mock(name='server')
        mock_wsgi_server.return_value = mock_server
        listen = [('192.0.2.1', '80')]

        self.mock_app = mock.Mock()

        self.service = designate_service.WSGIService(
            self.mock_app, 'test-wsgi-service', listen
        )
        mock_wsgi_server.assert_called_once()

        self.assertIsNone(self.service.start())
        mock_server.start.assert_called_once()

        self.assertIsNone(self.service.stop())
        mock_server.stop.assert_called_once()

    @mock.patch('oslo_service.wsgi.Server')
    def test_service_wait(self, mock_wsgi_server):
        mock_server = mock.Mock(name='server')
        mock_wsgi_server.return_value = mock_server
        self.mock_app = mock.Mock()
        listen = [('192.0.2.1', '80')]

        self.service = designate_service.WSGIService(
            self.mock_app, 'test-wsgi-service', listen
        )
        mock_wsgi_server.assert_called_once()

        self.assertIsNone(self.service.wait())
        mock_server.wait.assert_called_once()

    @mock.patch('oslo_service.wsgi.Server')
    def test_service_wait_multiple_servers(self, mock_wsgi_server):
        mock_server = mock.Mock(name='server')
        mock_wsgi_server.return_value = mock_server
        self.mock_app = mock.Mock()
        listen = [('192.0.2.1', '80'), ('192.0.2.2', '80')]

        self.service = designate_service.WSGIService(
            self.mock_app, 'test-wsgi-service', listen
        )
        mock_wsgi_server.assert_called()
        self.assertEqual(2, mock_wsgi_server.call_count)

        self.assertIsNone(self.service.wait())
        mock_server.wait.assert_called()
        self.assertEqual(2, mock_server.wait.call_count)


class TestDNSService(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)

        self.tg = mock.Mock()
        self.storage = mock.Mock()
        self.application = handler.RequestHandler(self.storage, self.tg)

        self.service = designate_service.DNSService(
            self.application, self.tg,
            CONF['service:mdns'].listen,
            CONF['service:mdns'].tcp_backlog,
            CONF['service:mdns'].tcp_keepidle,
            CONF['service:mdns'].tcp_recv_timeout,
        )
        self.service._running = mock.Mock()

    def test_service_init(self):
        self.assertEqual(CONF['service:mdns'].listen, self.service.listen)
        self.assertEqual(
            CONF['service:mdns'].tcp_backlog, self.service.tcp_backlog
        )
        self.assertEqual(
            CONF['service:mdns'].tcp_keepidle, self.service.tcp_keepidle
        )
        self.assertEqual(
            CONF['service:mdns'].tcp_recv_timeout,
            self.service.tcp_recv_timeout
        )

    @mock.patch.object(utils, 'bind_tcp')
    @mock.patch.object(utils, 'bind_udp')
    def test_service_start(self, mock_bind_udp, mock_bind_tcp):
        self.service.start()

        mock_bind_udp.assert_called_with('0.0.0.0', 5354)
        mock_bind_tcp.assert_called_with(
            '0.0.0.0', 5354, CONF['service:mdns'].tcp_backlog,
            CONF['service:mdns'].tcp_keepidle,
        )

    def test_service_stop(self):
        mock_sock_tcp = mock.Mock()
        mock_sock_udp = mock.Mock()

        self.service._dns_socks_tcp = [mock_sock_tcp]
        self.service._dns_socks_udp = [mock_sock_udp]

        self.service.stop()

        mock_sock_tcp.close.assert_called_once()
        mock_sock_udp.close.assert_called_once()

    def test_handle_tcp(self):
        self.service._running.is_set.side_effect = [True, True, False]

        mock_client = mock.Mock()
        addr = ('192.0.2.1', 5353, '203.0.113.2', 5353)

        mock_sock_tcp = mock.Mock()
        mock_sock_tcp.accept.return_value = (mock_client, addr)

        self.assertIsNone(self.service._dns_handle_tcp(mock_sock_tcp))

        mock_sock_tcp.accept.assert_called()
        mock_client.settimeout.assert_called()

    def test_handle_tcp_handle_errors(self):
        self.service._running.is_set.side_effect = [True, True, True, False]

        mock_sock_tcp = mock.Mock()
        mock_sock_tcp.accept.side_effect = [
            socket.timeout(), socket.error(errno.EACCES), Exception()
        ]

        self.assertIsNone(self.service._dns_handle_tcp(mock_sock_tcp))

        mock_sock_tcp.accept.assert_called()

    def test_handle_tcp_os_error(self):
        self.service._running.is_set.side_effect = [True, True, True, False]

        mock_socket = mock.Mock()
        mock_sock_tcp = mock.Mock()
        mock_sock_tcp.accept.return_value = (mock_socket, None,)
        mock_socket.settimeout.side_effect = [
            OSError(errno.EACCES), Exception()
        ]

        self.assertIsNone(self.service._dns_handle_tcp(mock_sock_tcp))

        mock_sock_tcp.accept.assert_called()

    def test_handle_tcp_without_timeout(self):
        CONF.set_override('tcp_recv_timeout', 0, 'service:mdns')

        self.service = designate_service.DNSService(
            self.application, self.tg,
            CONF['service:mdns'].listen,
            CONF['service:mdns'].tcp_backlog,
            CONF['service:mdns'].tcp_keepidle,
            CONF['service:mdns'].tcp_recv_timeout,
        )
        self.service._running = mock.Mock()
        self.service._running.is_set.side_effect = [True, True, False]

        mock_client = mock.Mock()
        addr = ('192.0.2.1', 5353)

        mock_sock_tcp = mock.Mock()
        mock_sock_tcp.accept.return_value = (mock_client, addr)

        self.assertIsNone(self.service._dns_handle_tcp(mock_sock_tcp))

        mock_sock_tcp.accept.assert_called()
        mock_client.settimeout.assert_not_called()

    def test_handle_tcp_running_not_set(self):
        mock_sock_tcp = mock.Mock()
        self.service._running.is_set.side_effect = [False]
        self.assertIsNone(self.service._dns_handle_tcp(mock_sock_tcp))

    def test_handle_udp(self):
        self.service._running.is_set.side_effect = [True, True, False]

        mock_client = mock.Mock()
        addr = ('192.0.2.1', 5353)

        mock_sock_udp = mock.Mock()
        mock_sock_udp.recvfrom.return_value = (mock_client, addr)

        self.assertIsNone(self.service._dns_handle_udp(mock_sock_udp))

        mock_sock_udp.recvfrom.assert_called()

    def test_handle_udp_handle_errors(self):
        self.service._running.is_set.side_effect = [True, True, True, False]

        mock_sock_tcp = mock.Mock()
        mock_sock_tcp.recvfrom.side_effect = [
            socket.timeout(), socket.error(errno.EACCES), Exception()
        ]

        self.assertIsNone(self.service._dns_handle_udp(mock_sock_tcp))

        mock_sock_tcp.recvfrom.assert_called()

    def test_dns_handle_udp_query(self):
        mock_sock = mock.Mock()

        addr = ('192.0.2.1', 53)

        self.service.app = mock.Mock()
        self.service.app.return_value = [None, 'response']

        self.service._dns_handle_udp_query(mock_sock, addr, 'payload')

        mock_sock.sendto.assert_called_with('response', addr)

    def test_dns_handle_tcp_conn_expected_length_raw_zero(self):
        mock_client = mock.Mock()
        mock_client.recv.return_value = []
        addr = ('192.0.2.1', 53)

        self.assertIsNone(self.service._dns_handle_tcp_conn(addr, mock_client))

        mock_client.recv.assert_called_with(2)

    def test_dns_handle_tcp_conn_buffer_empty(self):
        mock_client = mock.Mock()
        mock_client.recv.side_effect = (struct.pack('!H', 2), None)
        addr = ('192.0.2.1', 53)

        self.service.app = mock.Mock()
        self.service.app.return_value = [None]

        self.service._dns_handle_tcp_conn(addr, mock_client)

        mock_client.recv.assert_called_with(2)

    def test_dns_handle_tcp_conn_no_app_response(self):
        mock_client = mock.Mock()
        mock_client.recv.side_effect = (struct.pack('!H', 2), b'buffer', b'')
        addr = ('192.0.2.1', 53)

        self.service.app = mock.Mock()
        self.service.app.return_value = [None]

        self.service._dns_handle_tcp_conn(addr, mock_client)

        mock_client.recv.assert_called_with(2)

    @mock.patch('struct.unpack')
    def test_dns_handle_tcp_conn_struct_error(self, mock_struct_unpack):
        mock_client = mock.Mock()
        mock_struct_unpack.side_effect = struct.error()
        mock_client.recv.side_effect = ('bad data',)
        addr = ('192.0.2.1', 53)

        self.service.app = mock.Mock()
        self.service.app.return_value = [None]

        self.service._dns_handle_tcp_conn(addr, mock_client)

        mock_client.recv.assert_called_with(2)

        self.assertIn(
            'Invalid packet from: 192.0.2.1:53',
            self.stdlog.logger.output
        )



# Copyright (c) 2014 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from designate.objects import base
from designate.objects import fields
from designate.objects.record import Record
from designate.objects.record import RecordList


@base.DesignateRegistry.register
class SSHFP(Record):
    """
    SSHFP Resource Record Type
    Defined in: RFC4255
    """
    fields = {
        'algorithm': fields.IntegerFields(minimum=0, maximum=4),
        'fp_type': fields.IntegerFields(minimum=0, maximum=2),
        'fingerprint': fields.Sshfp(nullable=True),
    }

    def from_string(self, value):
        algorithm, fp_type, fingerprint = value.split(' ')

        for value in {algorithm, fp_type}:
            if repr(int(value)) != value:
                raise ValueError('Value is not an integer')

        self.algorithm = int(algorithm)
        self.fp_type = int(fp_type)
        self.fingerprint = fingerprint

    # The record type is defined in the RFC. This will be used when the record
    # is sent by mini-dns.
    RECORD_TYPE = 44


@base.DesignateRegistry.register
class SSHFPList(RecordList):

    LIST_ITEM_TYPE = SSHFP
    fields = {
        'objects': fields.ListOfObjectsField('SSHFP'),
    }
